==> hermes-6.1.0.18.rec2.tar.gz: hermes-6.1.0.18.rec2/app/controllers/hermes/api/dmo/application_controller.rb <==
# frozen_string_literal: true
module Hermes
  module Api
    module Dmo
      # Base Controller for Hermes app (Parent Class)
      class ApplicationController < ::Api::D2::ApplicationController
        # protect_from_forgery with: :exception
        @@engine_path = Hermes::Engine.root
        require_sugar_tree = %w[lib hermes tools require_sugar]

        require @@engine_path.join(*require_sugar_tree).to_s

        serialization_scope :current_user
        respond_to :json

        @@base_set_instance_for = %i[show update destroy]

        before_action :allow_true_user, :check_if_log_as, if: :true_user?
        after_filter :log_action

        def log_action
          status = response.code
          p = params.except(:controller, :uuid, :doc, :space, :auth_token, :file, :action)
          Hermes::LogJob.add_to_queue(
            user_id: (current_user && current_user.id.to_s), 
            code: "API-#{controller_path}", 
            action: "verb : #{env["REQUEST_METHOD"]} | status : #{status} | #{params[:action]}",
            request: request_info,
            d: p,
          )
        end

        def initialize
          @query_precision = {}
          @options = {}
          @module_name = 'Hermes'
          super
          init_params_and_operators
        end

        class << self
          def set_instance_for(*arr)
            @set_instance_for ||= @@base_set_instance_for
            @set_instance_for = @set_instance_for.concat(arr).uniq.compact
          end
        end

        def mutate_params          
          raise Exception, "Missing mutate_params method for controller #{self.class}"
        end 

        def init_params_and_operators
          @additional_params = {}
          @acceptable_params = [:created_at, :updated_at]
          @regex_default_params = @regex_params = @valid_operators = []
        end

        before_action :set_instance, only: @set_instance_for, unless: :skip_set_instance?

        def handle_order_param(val)
          return unless [1, -1, '1', '-1', 'asc', 'desc', :asc, :desc].include?(val)

          val = Integer(val) if ['1', '-1'].include?(val)
          if [1,-1].include? val
            val == 1 ? :asc : :desc
          elsif %w[asc desc].include?(val)
            val.to_sym
          else
            val
          end
        end

        def order_params
          if %w[
            ActionController::Parameters
            Mongoid::Document
            Hash
          ].include?(params[:sort].class.name)
            params[:sort].each_with_object({}) do |(key, val), h|
              next unless handle_order_param(val)

              h[key] = handle_order_param(val)
            end
          end
        end

        def include_relationships
          []
        end

        def index_aggregation(target_model: @model, no_acl: false)
          return unless target_model

          precision = @query_precision || {}
          precision[:user_id] = params[:responsible] if params[:responsible]
          query = target_model
          query = query.readable_by(@user) unless no_acl
          query.where(build_query(target_model).deep_merge(precision))
               .order_by(order_params)
               .includes(include_relationships)
        end

        # a utilser pour les parametres d'update et create
        def mutate_valid_params
          ActiveModelSerializers::Deserialization.jsonapi_parse!(
            params,
            only: self.mutate_params, key_transform: :unaltered
          ).merge(@additional_params)
        end

        def not_mutable
          ActiveModelSerializers::Deserialization.jsonapi_parse!(
            params,
            except: self.mutate_params, key_transform: :unaltered
          )
        end 

        # def render_error_not_mutable(not_mutable_fields)
        #   render json: {
        #     errors: not_mutable_fields.keys.map { |field| { "#{field}": "not mutable" }} , 
        #   },status: :unprocessable_entity
        # end 

        def index
          paginate(
            index_aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def show
          return render_json_error(:not_found, :object_not_found) unless @instance

          render(
            jsonapi: @instance,
            serializer: @serializer,
            **@options
          )
        end

        def create
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          create_without_render
          return respond_with_errors(@instance) unless @instance.save

          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end

        def create_without_render
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          Rails.logger.debug "TEST before #{user_id}"
          user_id = param_user if @user.has_role?('Administrator') && param_user
          Rails.logger.debug "TEST #{mutate_valid_params} after #{user_id}"

          @instance = @model.new(mutate_valid_params.merge(user_id: user_id))
        end

        def update
          return render_json_error(:not_found, :object_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :not_writers) unless @instance.writable_by?(@user)
          check_params_and_update
        end
        
        def check_params_and_update
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          valid_params = mutate_valid_params
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
          return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
          Rails.logger.debug "TEST valid_params are #{valid_params}"
          return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

          render(jsonapi: @instance, serializer: @serializer)
        end

        def user_can_destroy?
          is_writer_and_is_owner = (@instance.user_id.to_s == current_user.id.to_s) &&
            (current_user_roles & ['Manager', 'cre']).present?

          is_writer_and_is_owner || is_super_admin || current_user_roles.include?('Administrator')
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_found, :not_admin_or_manager) unless user_can_destroy?
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy

          render(body: nil, status: :no_content)
        end

        def set_instance
          @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
        end

        def allow_true_user
          allowed = ['superadministrateur', 'collaborateur offre logement'] & @true_user.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact
          @allow_true_user ||= allowed.any?

          # remove all controllers restrictions for switch delegation
          @allow_true_user = true
        end

        def current_user_roles
          current_user.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact
        end

        def current_user
          @user
        end

        def current_lessor
          (current_user_roles & ['bailleur']).present? ? current_user.lessor : nil
        end

        def per_page
          @per_apge ||= params[:per_page]&.to_i || 15
        end

        def page
          @page ||= (param_page = params[:page]).present? ? (p = param_page.to_i) <= 0 ? 1 : p : 1
        end

        def flatten_hash(param, prefix=nil)
          param.each_pair.reduce({}) do |a, (k, v)|
            v.is_a?(Hash) ? a.merge(flatten_hash(v, "#{prefix}#{k}.")) : a.merge("#{prefix}#{k}".to_sym => v)
          end
        end

        def is_super_admin
          current_user_roles.include?('superadministrateur')
        end

        def admin_search_param(p, keys_prefix, possible_params, full_regex_params = [], partial_regex_params = [])
          query = p.nil? ? {} : p.slice(*(possible_params+full_regex_params+partial_regex_params))
          query[:_id] = BSON::ObjectId(query.delete(:id)) if query.key?(:id)
          full_regex_params.each do |regex_param|
            query[regex_param.to_sym] = /^#{::Regexp.escape(query[regex_param.to_sym]).accent_to_regexp}$/i unless query[regex_param.to_sym].blank?
          end
          partial_regex_params.each do |regex_param|
            query[regex_param.to_sym] = /#{::Regexp.escape(query[regex_param.to_sym]).accent_to_regexp}/i unless query[regex_param.to_sym].blank?
          end
          query = query.transform_keys{ |key| keys_prefix.nil? ? key : "#{keys_prefix}.#{key}" } if query.length
        end

        def facet_count_query(params)
          page = params[:page]&.to_i || 1
          perPage = params[:per_page]&.to_i || 15
          skipPage = (page - 1) * perPage
          {
            :$facet => {
              :paginatedResults =>[{ :$skip  => skipPage }, { :$limit  => perPage }],
              :totalCount => [{ :$count => 'count' } ]
            }
          }
        end

        def paginate_like_array(collection, mapped_attributes = {}, facet_count = true)

          if facet_count
            results = collection.to_a[0]
            paginated_entries = results["paginatedResults"]
            count = (results["totalCount"].empty?) ? 0 : results["totalCount"][0]["count"]
          else
            count = collection.count
            paginated_entries = collection.paginate(per_page: per_page, page: page)
          end

          total_pages = (t_p = (count.to_f / per_page).ceil) >= 1 ? t_p : 1

          self_link = request.original_url.to_page(page)
          first_link = self_link.to_page(1)
          prev_link = (prev_p = page - 1).positive? ? self_link.to_page(prev_p) : nil
          next_link = (next_p = page + 1) > total_pages ? nil : self_link.to_page(next_p)
          last_link = self_link.to_page(total_pages)

          if mapped_attributes.length
            paginated_entries = paginated_entries.map do |e|
              data = {}
              mapped_attributes.each_key do |key|
                data[key] = eval("e#{mapped_attributes[key]}") rescue nil
              end
              data
            end
          end

          render jsonapi: {
            data: paginated_entries,
            links: {
              self: self_link,
              first: first_link,
              prev: prev_link,
              next: next_link,
              last: last_link
            },
            meta: {
              pagination: {
                per_page: per_page,
                total_pages: total_pages,
                total_objects: count,
                links: {
                  first: first_link[/\/api\/.*/],
                  last: last_link[/\/api\/.*/]
                }
              }
            }
          }
        end

        def paginate_with_same_order(collection, **options)
          entries_class = options[:entries_class] || @model
          serializer = options[:each_serializer] || @serializer

          paginated_entries = collection.paginate(per_page: per_page, page: page)
          count = collection.count

          total_pages = (t_p = (count.to_f / per_page).ceil) >= 1 ? t_p : 1
            
          self_link = request.original_url.to_page(page)
          first_link = self_link.to_page(1)
          prev_link = (prev_p = page - 1).positive? ? self_link.to_page(prev_p) : nil
          next_link = (next_p = page + 1) > total_pages ? nil : self_link.to_page(next_p)
          last_link = self_link.to_page(total_pages)

          fake_serial = paginated_entries.map do |object|
            next unless object
            
            serializer.new(
              object,
              scope: current_user,
              root: true,
              **options
            )
          end

          render jsonapi: {
            data: fake_serial.map do |entry|
              next unless entry
              
              {
                id: entry.id,
                type: entry._type,
                attributes: entry.attributes.reject { |key| ['id', :id].include? key }
              }
            end,
            links: {
              self: self_link,
              first: first_link,
              prev: prev_link,
              next: next_link,
              last: last_link
            },
            meta: {
              pagination: {
                per_page: per_page,
                total_pages: total_pages,
                total_objects: count,
                links: {
                  first: first_link[/\/api\/.*/],
                  last: last_link[/\/api\/.*/]
                }
              }
            }
          }

        end


        def paginate(collection, **options)
          if collection.is_a? Mongo::Collection::View::Aggregation
            identifier = options[:entries_identifier] || '_id'
            entries_class = options[:entries_class] || @model
            serializer = options[:each_serializer] || @serializer
            field_identifier = options[:field_identifier] || 'id'
            score_identifier = options[:score_identifier] || 'scores._id'
                      
            paginated_entries = collection.paginate(per_page: per_page, page: page)
            count = collection.count

            total_pages = (t_p = (count.to_f / per_page).ceil) >= 1 ? t_p : 1
            
            self_link = request.original_url.to_page(page)
            first_link = self_link.to_page(1)
            prev_link = (prev_p = page - 1).positive? ? self_link.to_page(prev_p) : nil
            next_link = (next_p = page + 1) > total_pages ? nil : self_link.to_page(next_p)
            last_link = self_link.to_page(total_pages)

            entries_ids = paginated_entries.map do |entry|
              identifier.split(".").inject(entry) { |hash, key| hash&.fetch(key, nil) }
            end

            all_entries = entries_class.where(field_identifier => { "$in" => entries_ids })

            fake_serial = entries_ids.map do |id|
              object = all_entries.detect { |e| e[field_identifier] == id }
              next unless object
              
              serializer.new(
                object,
                scope: current_user,
                root: true,
                **options
              )
            end

            render jsonapi: {
              data: fake_serial.map do |entry|
                next unless entry
                
                {
                  id: entry.id,
                  type: entry._type,
                  attributes: entry.attributes.reject { |key| ['id', :id].include? key }
                }
              end,
              links: {
                self: self_link,
                first: first_link,
                prev: prev_link,
                next: next_link,
                last: last_link
              },
              meta: {
                pagination: {
                  per_page: per_page,
                  total_pages: total_pages,
                  total_objects: count,
                  links: {
                    first: first_link[/\/api\/.*/],
                    last: last_link[/\/api\/.*/]
                  }
                }
              }
            }
          else
            super(collection, **options)
          end
        end

        def paginate_with_other_count(collection, **options)
          if options[:other_count]
            serializer = options[:each_serializer] || @serializer

            paginated_entries = collection.paginate(per_page: per_page, page: page)
            count = collection.count
            total_pages = (t_p = (count.to_f / per_page).ceil) >= 1 ? t_p : 1

            self_link = request.original_url.to_page(page)
            first_link = self_link.to_page(1)
            prev_link = (prev_p = page - 1).positive? ? self_link.to_page(prev_p) : nil
            next_link = (next_p = page + 1) > total_pages ? nil : self_link.to_page(next_p)
            last_link = self_link.to_page(total_pages)

            render jsonapi: {
              data: paginated_entries.map do |object|
                next unless object
              
                serializer.new(
                  object,
                  scope: current_user,
                  root: true,
                  **options
                )
              end,
              links: {
                self: self_link,
                first: first_link,
                prev: prev_link,
                next: next_link,
                last: last_link
              },
              meta: {
                pagination: {
                  per_page: per_page,
                  total_pages: total_pages,
                  total_objects: count,
                  links: {
                    first: first_link[/\/api\/.*/],
                    last: last_link[/\/api\/.*/]
                  },
                },
              }.merge(options[:other_count])
            }
          
          else
            paginate(collection, **options)
          end

        end 
        protected
        
        def true_user?
          @true_user.present?
        end

        def check_if_log_as
          unless @allow_true_user
            render json: {
              success: false,
              message: 'Action not allowed'
            }, status: :unauthorized
          end
        end

        def skip_set_instance?
          ['notifications', 'subscriptions'].include? controller_name
        end
      end
    end
  end
end

==> hermes-6.2.1.7.rec2.tar.gz: hermes-6.2.1.7.rec2/app/controllers/hermes/api/dmo/application_controller.rb <==
# frozen_string_literal: true
module Hermes
  module Api
    module Dmo
      # Base Controller for Hermes app (Parent Class)
      class ApplicationController < ::Api::D2::ApplicationController
        # protect_from_forgery with: :exception
        @@engine_path = Hermes::Engine.root
        require_sugar_tree = %w[lib hermes tools require_sugar]

        require @@engine_path.join(*require_sugar_tree).to_s

        serialization_scope :current_user
        respond_to :json

        @@base_set_instance_for = %i[show update destroy]

        before_action :allow_true_user, :check_if_log_as, if: :true_user?
        after_filter :log_action

        def log_action
          status = response.code
          p = params.except(:controller, :uuid, :doc, :space, :auth_token, :file, :action)
          Hermes::LogJob.add_to_queue(
            user_id: (current_user && current_user.id.to_s), 
            code: "API-#{controller_path}", 
            action: "verb : #{env["REQUEST_METHOD"]} | status : #{status} | #{params[:action]}",
            request: request_info,
            d: p,
          )
        end

        def initialize
          @query_precision = {}
          @options = {}
          @module_name = 'Hermes'
          super
          init_params_and_operators
        end

        class << self
          def set_instance_for(*arr)
            @set_instance_for ||= @@base_set_instance_for
            @set_instance_for = @set_instance_for.concat(arr).uniq.compact
          end
        end

        def mutate_params          
          raise Exception, "Missing mutate_params method for controller #{self.class}"
        end 

        def init_params_and_operators
          @additional_params = {}
          @acceptable_params = [:created_at, :updated_at]
          @regex_default_params = @regex_params = @valid_operators = []
        end

        before_action :set_instance, only: @set_instance_for, unless: :skip_set_instance?

        def handle_order_param(val)
          return unless [1, -1, '1', '-1', 'asc', 'desc', :asc, :desc].include?(val)

          val = Integer(val) if ['1', '-1'].include?(val)
          if [1,-1].include? val
            val == 1 ? :asc : :desc
          elsif %w[asc desc].include?(val)
            val.to_sym
          else
            val
          end
        end

        def order_params
          if %w[
            ActionController::Parameters
            Mongoid::Document
            Hash
          ].include?(params[:sort].class.name)
            params[:sort].each_with_object({}) do |(key, val), h|
              next unless handle_order_param(val)

              h[key] = handle_order_param(val)
            end
          end
        end

        def include_relationships
          []
        end

        def index_aggregation(target_model: @model, no_acl: false)
          return unless target_model

          precision = @query_precision || {}
          precision[:user_id] = params[:responsible] if params[:responsible]
          query = target_model
          query = query.readable_by(@user) unless no_acl
          query.where(build_query(target_model).deep_merge(precision))
               .order_by(order_params)
               .includes(include_relationships)
        end

        # a utilser pour les parametres d'update et create
        def mutate_valid_params
          ActiveModelSerializers::Deserialization.jsonapi_parse!(
            params,
            only: self.mutate_params, key_transform: :unaltered
          ).merge(@additional_params)
        end

        def not_mutable
          ActiveModelSerializers::Deserialization.jsonapi_parse!(
            params,
            except: self.mutate_params, key_transform: :unaltered
          )
        end 

        # def render_error_not_mutable(not_mutable_fields)
        #   render json: {
        #     errors: not_mutable_fields.keys.map { |field| { "#{field}": "not mutable" }} , 
        #   },status: :unprocessable_entity
        # end 

        def index
          paginate(
            index_aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def show
          return render_json_error(:not_found, :object_not_found) unless @instance

          render(
            jsonapi: @instance,
            serializer: @serializer,
            **@options
          )
        end

        def create
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          create_without_render
          return respond_with_errors(@instance) unless @instance.save

          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end

        def create_without_render
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          Rails.logger.debug "TEST before #{user_id}"
          user_id = param_user if @user.has_role?('Administrator') && param_user
          Rails.logger.debug "TEST #{mutate_valid_params} after #{user_id}"

          @instance = @model.new(mutate_valid_params.merge(user_id: user_id))
        end

        def update
          return render_json_error(:not_found, :object_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :not_writers) unless @instance.writable_by?(@user)
          check_params_and_update
        end
        
        def check_params_and_update
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          valid_params = mutate_valid_params
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
          return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
          Rails.logger.debug "TEST valid_params are #{valid_params}"
          return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

          render(jsonapi: @instance, serializer: @serializer)
        end

        def user_can_destroy?
          is_writer_and_is_owner = (@instance.user_id.to_s == current_user.id.to_s) &&
            (current_user_roles & ['Manager', 'cre']).present?

          is_writer_and_is_owner || is_super_admin || current_user_roles.include?('Administrator')
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_found, :not_admin_or_manager) unless user_can_destroy?
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy

          render(body: nil, status: :no_content)
        end

        def set_instance
          @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
        end

        def allow_true_user
          allowed = ['superadministrateur', 'collaborateur offre logement'] & @true_user.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact
          @allow_true_user ||= allowed.any?

          # remove all controllers restrictions for switch delegation
          @allow_true_user = true
        end

        def current_user_roles
          current_user.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact
        end

        def current_user
          @user
        end

        def current_lessor
          (current_user_roles & ['bailleur']).present? ? current_user.lessor : nil
        end

        def per_page
          @per_apge ||= params[:per_page]&.to_i || 15
        end

        def page
          @page ||= (param_page = params[:page]).present? ? (p = param_page.to_i) <= 0 ? 1 : p : 1
        end

        def flatten_hash(param, prefix=nil)
          param.each_pair.reduce({}) do |a, (k, v)|
            v.is_a?(Hash) ? a.merge(flatten_hash(v, "#{prefix}#{k}.")) : a.merge("#{prefix}#{k}".to_sym => v)
          end
        end

        def is_super_admin
          current_user_roles.include?('superadministrateur')
        end

        def admin_search_param(options)
          p = options[:params]
          possible_params = options[:possible_params] || []
          full_regex_params =  options[:full_regex_params] || []
          partial_regex_params = options[:partial_regex_params] || []

          query = p.nil? ? {} : p.slice(*(possible_params+full_regex_params+partial_regex_params))
          full_regex_params.each do |regex_param|
            query[regex_param.to_sym] = /^#{::Regexp.escape(query[regex_param.to_sym]).accent_to_regexp}$/i unless query[regex_param.to_sym].blank?
          end
          partial_regex_params.each do |regex_param|
            query[regex_param.to_sym] = /#{::Regexp.escape(query[regex_param.to_sym]).accent_to_regexp}/i unless query[regex_param.to_sym].blank?
          end
          query
        end

        def paginate(collection, **options)
          serializer = options[:each_serializer] || @serializer
          entries_class = options[:entries_class] || @model
          identifier = options[:entries_identifier] || '_id'
          field_identifier = options[:field_identifier] || 'id'

          if collection.is_a? Mongo::Collection::View::Aggregation
            count = collection.count

            entries_ids = collection.paginate(per_page: per_page, page: page).map do |entry|
              identifier.split(".").inject(entry) { |hash, key| hash&.fetch(key, nil) }
            end

            all_entries = entries_class.where(field_identifier => { "$in" => entries_ids }).limit(per_page)
            paginated_entries = []
            entries_ids.each do |id|
              paginated_entries.push(all_entries.detect { |e| e[field_identifier] == id })
            end 
          else
            paginated_entries = collection.hint(options[:hint]).paginate(per_page: per_page, page: page)
            count = collection.count(hint: options[:hint])
          end

          total_pages = (t_p = (count.to_f / per_page).ceil) >= 1 ? t_p : 1

          self_link = request.original_url.to_page(page)
          first_link = self_link.to_page(1)
          prev_link = (prev_p = page - 1).positive? ? self_link.to_page(prev_p) : nil
          next_link = (next_p = page + 1) > total_pages ? nil : self_link.to_page(next_p)
          last_link = self_link.to_page(total_pages)

          data = paginated_entries.map do |object|
            next unless object
          
            entry = serializer.new(
              object,
              scope: current_user,
              root: true,
              **options
            )

            json = {
              id: entry.id,
              type: entry._type,
            }

            attributes = entry.attributes.reject { |key| ['id', :id].include? key }
            options[:no_attributes] ? json = json.merge(attributes) : json[:attributes] = attributes
            json
          end

          response.headers['X-Total-Pages'] = total_pages
          response.headers['X-Total-Count'] = count
          response.headers['Link'] = "<#{first_link}>; rel=\"first\" , <#{last_link}>; rel=\"last\""
          response.headers['Link'] += ", <#{next_link}>; rel=\"next\"" if next_link

          render jsonapi: {
            data: data,
            links: {
              self: self_link,
              first: first_link,
              prev: prev_link,
              next: next_link,
              last: last_link
            },
            meta: {
              pagination: {
                per_page: per_page,
                total_pages: total_pages,
                total_objects: count,
                links: {
                  first: first_link[/\/api\/.*/],
                  last: last_link[/\/api\/.*/]
                },
              },
            }.merge(options[:other_count]&.symbolize_keys || {})
          }
        end 
        
        protected
        
        def true_user?
          @true_user.present?
        end

        def check_if_log_as
          unless @allow_true_user
            render json: {
              success: false,
              message: 'Action not allowed'
            }, status: :unauthorized
          end
        end

        def skip_set_instance?
          ['notifications', 'subscriptions'].include? controller_name
        end

        def set_user
          unless auth_user_token.blank?
            return if @user = User.find_by(:authentication_token => auth_user_token)
            return render status: :unauthorized, json: { errors: ['Wrong token'], success: false }
          end

          return render status: :unauthorized, json: { errors: ['Not Authenticated'], success: false } if bearer_token.blank?
          
          begin
            decoded_token = JWT.decode bearer_token, APP_CONFIG["token"], true, { algorithm: 'HS256' }
            infos = decoded_token.first["data"]
            @user = User.find_by(id: infos["id"])
            @true_user = User.find_by(id: infos["true_user_id"]) if infos["true_user_id"]
            user_token = Hermes::UserToken.find_by(user_id: @user.id, token: bearer_token)

            return render status: :unauthorized, json: { errors: ['User token is missing'], success: false } if user_token.nil?
          rescue JWT::ExpiredSignature
            return render status: :unauthorized, json: { errors: ['Token expired'], success: false}
          rescue => e
            return render status: :unauthorized, json: { errors: ['Unknown error'], success: false }
          end
        end

      end
    end
  end
end
